#ifndef LOGICPROGRAMPROBLEM_H_
#define LOGICPROGRAMPROBLEM_H_

#include <string>
#include <vector>
#include <map>
#include <ctime>

#include <sharp/main>

#include <parser/Driver.h>

#include <asp-domain/asp-domain.hpp>
#include <asp-domain/StringGenerator.hpp>

using namespace sharp;

namespace lpopt
{

	class LogicProgramProblem : public Problem
	{
	  friend class LogicProgramOptimizationAlgorithm; // may insert into createdProgam. May read program, currentStatement, ignoreHeadVariables. May write to width.
	  
	public:
		LogicProgramProblem(std::istream *stream, 
				AbstractHypertreeDecompositionAlgorithm *htdalg, 
				bool printBenchmarkInformation = false,
				bool ignoreHeadVariables = false);

		virtual ~LogicProgramProblem();

	public:
		virtual Solution *calculateSolution(AbstractHTDAlgorithm *algorithm);
		Program* getOptimizedProgram(); // caller has to delete. Call exactly once after calling calculateSolution().
		const Program* getBaseProgram(); // caller must not delete
		unsigned int getWidth() const; // after solution calculation, returns size of biggest bag.

	protected:
		virtual void parse();
		virtual void preprocess();
		virtual ExtendedHypertree *prepareHypertreeDecomposition(ExtendedHypertree *root);
		virtual Hypergraph *buildHypergraphRepresentation();

	private:
		parser::Driver *parsingDriver;
		Program *program;
		vector<Statement *> *createdProgram;
		unsigned int currentStatement; // the program's current statement in question
		const Statement& curStatement() const;
		StringGenerator* gen;
		clock_t htdlib_time, parse_time, me_time;
		const bool benchmark, ignoreHeadVariables;
		unsigned int width; // the decomposition's width - will be calculated when traversed (size of biggest bag)
	};
} // lpopt

#endif //DATALOGPROBLEM_H_
