#include <stdexcept>
#include <string>

#include "Driver.h"
#include "scanner.h"

namespace parser {

Driver::Driver(std::istream *stream) : stream(stream), program(NULL)
{
}

Driver::~Driver()
{
}

domain::Program* Driver::parse()
{
	delete program;
	FlexScanner scanner(stream);
	::parser::Parser parser(*this, scanner);
	int res = parser.parse();
	if (res == 0) return program;
	throw std::runtime_error(std::string("Parse error. Parser returned ") + std::to_string(res));
}

void Driver::error(const parser::location& l, const std::string& m)
{
	std::ostringstream ss;
	ss << "Parse error." << std::endl << l << ": " << m;
	throw std::runtime_error(ss.str());
}

} // namespace parser
